@@ -1,6 +1,10 @@
 from dotenv import load_dotenv
 import os
 from decimal import Decimal
+
+from ostium_python_sdk.utils import calculate_fee_per_hours, convert_decimals, format_with_precision
+
+from .formulae_wrapper import get_funding_fee_long_short, get_trade_metrics
 from .constants import PRECISION_2, PRECISION_6, PRECISION_12, PRECISION_18, PRECISION_9
 
 from ostium_python_sdk.faucet import Faucet
@@ -14,7 +18,8 @@ from .subgraph import SubgraphClient
 
 
 class OstiumSDK:
-    def __init__(self, network: Union[str, NetworkConfig], private_key: str = None, rpc_url: str = None):
+    def __init__(self, network: Union[str, NetworkConfig], private_key: str = None, rpc_url: str = None, verbose=False):
+        self.verbose = verbose
         load_dotenv()
         self.private_key = private_key or os.getenv('PRIVATE_KEY')
         # if not self.private_key:
@@ -52,20 +57,83 @@ class OstiumSDK:
             self.network_config.contracts["usdc"],
             self.network_config.contracts["tradingStorage"],
             self.network_config.contracts["trading"],
-            private_key=self.private_key
+            private_key=self.private_key,
+            verbose=self.verbose
         )
 
         # Initialize subgraph client
-        self.subgraph = SubgraphClient(url=self.network_config.graph_url)
+        self.subgraph = SubgraphClient(
+            url=self.network_config.graph_url, verbose=self.verbose)
 
-        self.balance = Balance(self.w3, self.network_config.contracts["usdc"])
-        self.price = Price()
+        self.balance = Balance(
+            self.w3, self.network_config.contracts["usdc"], verbose=self.verbose)
+        self.price = Price(verbose=self.verbose)
 
         if self.network_config.is_testnet:
-            self.faucet = Faucet(self.w3, self.private_key)
+            self.faucet = Faucet(self.w3, self.private_key,
+                                 verbose=self.verbose)
         else:
             self.faucet = None
 
+    def log(self, message):
+        if self.verbose:
+            print(message)
+
+    # if SDK instantiated with a private key, this function will return a given open trade metrics,
+    # such as: funding fee, roll over fee, Unrealized Pnl, Profit Percent, etc.
+    #
+    # Will thorw in case SDK instantiated with no private key
+    async def get_open_trade_metrics(self, pair_id, trade_index):
+        trader_public_address = self.ostium.get_public_address()
+        self.log(f"Trader public address: {trader_public_address}")
+        open_trades = await self.subgraph.get_open_trades(trader_public_address)
+
+        trade_details = None
+
+        if len(open_trades) == 0:
+            raise ValueError(f"No Open Trades for {trader_public_address}")
+
+        for t in open_trades:
+            if int(t['pair']['id']) == int(pair_id) and int(t['index']) == int(trade_index):
+                trade_details = t
+                break
+
+        if trade_details is None:
+            raise ValueError(
+                f"Trade not found for {trader_public_address} pair {pair_id} and index {trade_index}")
+
+        self.log(f"Trade details: {trade_details}")
+        # get the price for this trade's asset/feed
+        price_data = await self.price.get_latest_price_json(trade_details['pair']['from'], trade_details['pair']['to'])
+        self.log(f"Price data: {price_data} (need here bid, mid, ask prices)")
+        # get the block number
+        block_number = self.ostium.get_block_number()
+        self.log(f"Block number: {block_number}")
+        return get_trade_metrics(trade_details, price_data, block_number)
+
+    async def get_pair_net_rate_percent_per_hours(self, pair_id, period_hours=24):
+        pair_details = await self.subgraph.get_pair_details(pair_id)
+        block_number = self.ostium.get_block_number()
+
+        funding_fee_long_per_block, funding_fee_short_per_block = get_funding_fee_long_short(
+            pair_details, block_number)
+        rollover_fee_per_block = Decimal(
+            pair_details['rolloverFeePerBlock']) / Decimal('1e18')
+
+        ff_long = calculate_fee_per_hours(
+            funding_fee_long_per_block, hours=period_hours)
+        ff_short = calculate_fee_per_hours(
+            funding_fee_short_per_block, hours=period_hours)
+        rollover = calculate_fee_per_hours(
+            rollover_fee_per_block, hours=period_hours)
+
+        rollover_value = Decimal('0') if rollover == 0 else rollover
+        net_long_percent = format_with_precision(
+            ff_long-rollover_value, precision=4)
+        net_short_percent = format_with_precision(
+            ff_short-rollover_value, precision=4)
+        return net_long_percent, net_short_percent
+
     async def get_formatted_pairs_details(self) -> list:
         """
         Get formatted details for all trading pairs, with proper decimal conversion.
